#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2017 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#

# -------------------------------------------------------------------------------------
# 
# This Version - Version 2.08
# Change Log
# Version 1.00 Joined-up writing Primer
# The idea of this plugin is to take some text and convert it into a series of unicode characters
# and display the text as joined up characters using a custom unicode font.
# Version 1.02 work on a group of similar Joiner fonts with letter variations.
# Version 1.03 adds letter variations
# There are a number of letter variation options (currently 8 of these) to tweak the font style output.
# Version 1.04 adds a new font - ItalicVariations
# The range of joins has been increased from 4 to 6: for letter e, for rounded letters like a, for slanted letters like v, for straight letters like i, for letter s, for letter z.
# Version 1.05 adds a new font weight - ItalicBoldVariations
# Smart quotes added: Kerning added to all fonts (although this is largely irrelevant in joined-up mode): Fixed letter s issue to force correct join with italic fonts and alternative letter s.
# Version 1.06 adds a new font weight - ItalicBlackVariations: Kerning added to non-joining lower-case letters.
#
# Version 2.00 Major update to place joined-up text on the given path and produces a non-text layer. Where there is no path available the filter will display the joined-up text as a text layer.
# Version 2:01 Fixed Retain Layers when set to yes bug
# Version 2:02 Added new font "SquareBeads" to list of fonts
# Version 2:03 Added unicode 'utf-8' encoding for text entry to display copyright sign, pound sign and division sign characters in fonts
# Version 2:04 Stopped filter from creating unnecessary layers at very small, or very large, sizes
# Version 2:05 Added text size modifiers option to increase and decrease the size of text along the path
# Version 2:06 Added shaded text option to change the colour of text along the path
# Version 2:07 Added coloured text option to change the colour of text along the path using the colours from the ui gradient tool
# Version 2:08 No longer continues to write characters beyond the length of the active path; kerning adjustments for more letter layers added; more glyphs added to fonts
# 
# The plugin can be found via Filters>Text FX>Joined-up Text...
# If you have an older version of the plugin it is probabaly best to remove it or rename it and relocate it
#
# The filter uses the function "computeThetaWithSlope" written by Ofnuts
# and parts of code taken from Tranductin's filter, custom-font_on_path, which he attributes to a post on http://stackoverflow.com/questions/31735499/calculate-angle-clockwise-between-two-points
# Without their work I would not have been able to create this filter

# Import Gimp Python modules

from array import array
from gtk import gdk
from gimpfu import *
import math	

# Define some global variables

w = gdk.screen_width()
h = gdk.screen_height()

# Define some colours we may need to use

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)

# Define strings containing joining letters

a_joins = unichr(256)+unichr(257)+unichr(258)+unichr(259)+unichr(260)+unichr(261)
c_joins = unichr(268)+unichr(269)+unichr(270)+unichr(271)+unichr(272)+unichr(273)
d_joins = unichr(274)+unichr(275)+unichr(276)+unichr(277)+unichr(278)+unichr(279)
e_joins = unichr(280)+unichr(281)+unichr(282)+unichr(283)+unichr(284)+unichr(285)+unichr(286)+unichr(287)+unichr(288)+unichr(289)+unichr(290)+unichr(291)+unichr(292)+unichr(293)
g_joins = unichr(299)+unichr(300)+unichr(301)+unichr(302)+unichr(303)+unichr(421)+unichr(422)+unichr(423)+unichr(424)+unichr(425)+unichr(426)+unichr(427)
o_joins = unichr(345)+unichr(346)+unichr(347)+unichr(348)+unichr(349)
q_joins = unichr(356)+unichr(357)+unichr(358)+unichr(359)+unichr(360)+unichr(361)
lower_case_1 = "acdegoq"+a_joins+c_joins+d_joins+e_joins+g_joins+o_joins+q_joins
b_joins = unichr(262)+unichr(263)+unichr(264)+unichr(265)+unichr(266)+unichr(267)
h_joins = unichr(304)+unichr(305)+unichr(306)+unichr(307)+unichr(308)+unichr(309)
i_joins = unichr(310)+unichr(311)+unichr(312)+unichr(313)+unichr(314)+unichr(315)
k_joins = unichr(321)+unichr(322)+unichr(323)+unichr(324)+unichr(325)+unichr(326)+unichr(435)+unichr(436)+unichr(437)+unichr(438)+unichr(439)+unichr(440)+unichr(441)
l_joins = unichr(327)+unichr(328)+unichr(329)+unichr(330)+unichr(331)+unichr(332)
lower_case_2 = "bhikl"+b_joins+h_joins+i_joins+k_joins+l_joins
m_joins = unichr(333)+unichr(334)+unichr(335)+unichr(336)+unichr(337)+unichr(338)
n_joins = unichr(339)+unichr(340)+unichr(341)+unichr(342)+unichr(343)+unichr(344)
r_joins = unichr(362)+unichr(363)+unichr(364)+unichr(365)+unichr(366)
u_joins = unichr(379)+unichr(380)+unichr(381)+unichr(382)+unichr(383)+unichr(384)
y_joins = unichr(400)+unichr(401)+unichr(402)+unichr(403)+unichr(404)+unichr(405)+unichr(463)+unichr(464)+unichr(465)+unichr(466)+unichr(467)+unichr(468)+unichr(469)
lower_case_3 = "mnruy"+m_joins+n_joins+r_joins+u_joins+y_joins
v_joins = unichr(385)+unichr(386)+unichr(387)+unichr(388)+unichr(389)
w_joins = unichr(390)+unichr(391)+unichr(392)+unichr(393)+unichr(394)
lower_case_4 = "vw"+v_joins+w_joins
x_joins = unichr(395)+unichr(396)+unichr(397)+unichr(398)+unichr(399)
z_joins = unichr(405)+unichr(406)+unichr(407)+unichr(408)+unichr(409)+unichr(410)
lower_case_5 = "xz"+x_joins+z_joins
alt_x_joins = unichr(456)+unichr(457)+unichr(458)+unichr(459)+unichr(460)+unichr(461)+unichr(462)
lower_case_6 = alt_x_joins
alt_z_joins = unichr(470)+unichr(471)+unichr(472)+unichr(473)+unichr(474)+unichr(475)+unichr(476)
lower_case_7 = alt_z_joins
s_joins = unichr(367)+unichr(368)+unichr(369)+unichr(370)+unichr(371)+unichr(372)+unichr(442)+unichr(443)+unichr(444)+unichr(445)+unichr(446)+unichr(447)+unichr(448)+unichr(449)+unichr(450)+unichr(451)+unichr(452)+unichr(453)+unichr(454)+unichr(455)
lower_case_8 = "s"+s_joins

# horizontal adjustment information for layers created by characters with negative font positions ie fjp and t
f_alone_left_bearing = [208,216,216,280,0] # letter f standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
f_alone_right_bearing = [-190,-204,-188,-236,0] # letter f standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
f_left_bearing = [208,216,216,280,0] # letter f  x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
f_right_bearing = [-190,-204,-188,-236,0] # letter f  x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

alt_f_alone_left_bearing = [48,64,48,80,0] # alternative letter f standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_alone_right_bearing = [-32,-32,-16,-48,0] # alternative letter f standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_left_bearing = [48,64,48,80,0] # alternative letter f  x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_right_bearing = [-20,-32,-16,-48,0] # alternative letter f  x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
j_alone_left_bearing = [220,232,226,284,0] # letter j standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
j_alone_right_bearing = [-220,-232,-240,-280,0] # letter j standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
j_left_bearing = [220,232,226,284,0] # letter j x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
j_right_bearing = [-220,-232,-240,-280,0] # letter j x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

alt_j_alone_left_bearing = [248,276,248,284,0] # letter j standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_alone_right_bearing = [-240,-256,-256,-216,0] # letter j standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_left_bearing = [248,276,248,284,0] # letter j x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_right_bearing = [-240,-256,-256,-216,0] # letter j x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
p_alone_left_bearing = [160,176,172,0,0] # letter p standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
p_alone_right_bearing = [-152,-152,-148,0,0] # letter p standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
p_left_bearing = [160,176,172,0,0] # letter p x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
p_right_bearing = [-152,-152,-148,0,0] # letter p x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
t_alone_left_bearing = [44,52,48,60,0] # letter t standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
t_alone_right_bearing = [-28,-36,-24,-28,0] # letter t standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
t_left_bearing = [44,52,48,60,0] # letter t x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
t_right_bearing = [-28,-36,-24,-28,0] # letter t x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

# horizontal adjustment information for layers following those created by Capital letters 

A_to_ABDEFHKLMNPRS_right_bearing = [0,0,0,60,0]
A_to_J_right_bearing = [203,203,203,80,0]
A_to_CGOQ_right_bearing = [0,0,0,100,0]
A_to_T_right_bearing = [203,203,203,180,0]
A_to_VW_right_bearing = [104,159,145,200,0]
A_to_IU_right_bearing = [0,0,0,50,0]
A_to_Y_right_bearing = [67,94,67,180,0]
A_to_Z_right_bearing = [67,94,67,60,0]
A_to_lc1_right_bearing = [58,31,17,60,0]
A_to_lc2_3_right_bearing = [45,25,14,0,0]
A_to_lc4_right_bearing = [113,113,86,118,0]
A_to_lc5_right_bearing = [45,94,14,0,0]
A_to_lc6_right_bearing = [81,90,69,48,0]
A_to_lc7_right_bearing = [145,130,110,145,0]
A_to_lc8_right_bearing = [83,60,55,109,0]

B_to_A_right_bearing = [0,0,42,80,0]
B_to_BDEFLRS_right_bearing = [0,0,62,27,0]
B_to_CGOQ_right_bearing = [0,0,0,60,0]
B_to_I_right_bearing = [0,0,0,80,0]
B_to_T_right_bearing = [0,0,0,120,0]
B_to_VW_right_bearing = [0,0,73,100,0]
B_to_XYZ_right_bearing = [0,0,51,100,0]
B_to_lc4_right_bearing = [0,0,0,80,0]
B_to_lc5_right_bearing = [0,0,42,50,0]
B_to_lc6_lc7_right_bearing = [70,0,56,0,0]
B_to_lc8_right_bearing = [56,0,56,50,0]

C_to_A_right_bearing = [52,107,134,100,0]
C_to_BDEFLR_right_bearing = [0,63,77,50,0]
C_to_IKMNPU_right_bearing = [0,0,0,50,0]
C_to_CGOQS_right_bearing = [0,0,73,60,0]
C_to_M_right_bearing = [0,78,105,0,0]
C_to_JT_right_bearing = [0,85,126,80,0]
C_to_VW_right_bearing = [0,0,0,100,0]
C_to_XYZ_right_bearing = [54,54,95,100,0]
C_to_lc1_right_bearing = [55,100,82,0,0]
C_to_lc2_right_bearing = [0,50,55,0,0]
C_to_lc3_right_bearing = [0,50,82,0,0]
C_to_lc4_right_bearing = [51,55,82,0,0]
C_to_lc5_right_bearing = [55,55,82,30,0]
C_to_lc6_right_bearing = [75,82,123,0,0]
C_to_lc7_right_bearing = [100,123,137,0,0]
C_to_lc8_right_bearing = [75,82,109,60,0]

D_to_A_right_bearing = [82,82,109,100,0]
D_to_BDEFLR_right_bearing = [0,99,72,50,0]
D_to_HKNPUCGOQ_right_bearing = [0,0,0,50,0]
D_to_I_right_bearing = [0,0,0,60,0]
D_to_J_right_bearing = [219,137,123,60,0]
D_to_M_right_bearing = [0,86,72,50,0]
D_to_S_right_bearing = [85,85,85,50,0]
D_to_T_right_bearing = [66,66,121,150,0]
D_to_VW_right_bearing = [0,55,55,0,0]
D_to_XYZ_right_bearing = [109,109,136,100,0]
D_to_lc1_right_bearing = [55,0,41,0,0]
D_to_lc8_right_bearing = [78,41,54,0,0]

E_to_A_right_bearing = [0,55,28,50,0]
E_to_VWMXY_right_bearing = [0,0,0,50,0]
E_to_CGOQS_right_bearing = [0,0,62,80,0]
E_to_JT_right_bearing = [86,86,110,50,0]
E_to_lc1_right_bearing = [0,0,0,40,0]
E_to_lc4_right_bearing = [96,82,69,40,0]
E_to_lc6_right_bearing = [55,27,55,0,0]
E_to_lc7_right_bearing = [110,82,124,0,0]
E_to_lc8_right_bearing = [55,27,28,30,0]

F_to_A_right_bearing = [187,187,255,150,0]
F_to_J_right_bearing = [200,200,200,200,0]
F_to_M_right_bearing = [94,94,121,40,0]
F_to_BDEHLNU_right_bearing = [98,98,100,40,0]
F_to_CGOQ_right_bearing = [98,98,125,90,0]
F_to_FIKPRVW_right_bearing = [50,50,90,40,0]
F_to_S_right_bearing = [140,140,140,90,0]
F_to_T_right_bearing = [73,73,100,40,0]
F_to_XYZ_right_bearing = [85,85,100,60,0]
F_to_lc1_right_bearing = [186,186,200,80,0]
F_to_lc2_right_bearing = [40,40,63,0,0]
F_to_lc3_right_bearing = [137,110,137,50,0]
F_to_lc4_right_bearing = [117,117,117,40,0]
F_to_lc5_right_bearing = [125,100,125,40,0]
F_to_lc6_right_bearing = [125,83,125,40,0]
F_to_lc7_right_bearing = [109,83,137,40,0]
F_to_lc8_right_bearing = [193,186,213,60,0]

G_to_A_right_bearing = [0,59,155,100,0]
G_to_M_right_bearing = [0,59,155,40,0]
G_to_BDEFHIKLMNPRUS_right_bearing = [0,0,0,40,0]
G_to_J_right_bearing = [140,117,170,100,0]
G_to_T_right_bearing = [76,117,158,100,0]
G_to_XYZ_right_bearing = [0,67,136,100,0]
G_to_VW_right_bearing = [0,0,0,100,0]
G_to_lc1_right_bearing = [40,90,99,50,0]
G_to_lc2_lc3_right_bearing = [0,0,41,0,0]
G_to_lc4_lc5_right_bearing = [0,0,25,0,0]
G_to_lc6_right_bearing = [0,-53,43,0,0]
G_to_lc7_right_bearing = [0,-66,43,0,0]
G_to_lc8_right_bearing = [0,90,98,50,0]

H_to_A_right_bearing = [52,52,52,40,0]
H_to_J_right_bearing = [100,100,100,20,0]
H_to_M_right_bearing = [0,93,38,20,0]
H_to_T_right_bearing = [50,47,20,0,0]
H_to_VWXYZ_right_bearing = [0,0,0,40,0]
H_to_BDEFHIKLNPRU_right_bearing = [0,0,0,20,0]
H_to_COGQ_right_bearing = [0,0,0,20,0]
H_to_lc1_right_bearing = [46,41,13,0,0]
H_to_lc2_lc3_right_bearing = [41,41,13,0,0]
H_to_lc4_lc5_right_bearing = [41,41,13,0,0]
H_to_lc6_lc7_right_bearing = [0,38,13,0,0]
H_to_lc8_right_bearing = [67,65,40,40,0]

I_to_AM_right_bearing = [0,0,57,40,0]
I_to_B_right_bearing = [60,40,44,40,0]
I_to_CGOQ_right_bearing = [0,0,0,60,0]
I_to_DEFHIKLNPRUS_right_bearing = [0,0,0,40,0]
I_to_J_right_bearing = [104,104,104,60,0]
I_to_T_right_bearing = [60,60,87,40,0]
I_to_VW_right_bearing = [0,0,69,40,0]
I_to_XYZ_right_bearing = [0,0,0,40,0]
I_to_lc1_right_bearing = [40,40,40,50,0]
I_to_lc4_right_bearing = [40,40,55,40,0]
I_to_lc6_right_bearing = [0,28,42,0,0]
I_to_lc7_right_bearing = [0,41,77,0,0]
I_to_lc8_right_bearing = [0,14,42,50,0]

J_to_A_right_bearing = [0,0,0,120,0]
J_to_BDEFHIKLMNPRTUXYZ_right_bearing = [0,0,0,40,0]
J_to_CGOQS_right_bearing = [0,0,0,100,0]
J_to_VW_right_bearing = [0,0,0,50,0]
J_to_lc1_right_bearing = [47,47,27,80,0]
J_to_lc2_right_bearing = [40,22,12,0,0]
J_to_lc3_right_bearing = [40,26,16,0,0]
J_to_lc4_lc5_right_bearing = [40,41,11,50,0]
J_to_lc6_lc7_right_bearing = [49,41,21,50,0]
J_to_lc8_right_bearing = [88,47,47,80,0]

K_to_AUVWXY_right_bearing = [40,40,40,60,0]
K_to_BDEFHIKLMNPRZ_right_bearing = [0,0,40,40,0]
K_to_CGOQ_right_bearing = [60,60,60,120,0]
K_to_J_right_bearing = [152,152,152,100,0]
K_to_S_right_bearing = [0,0,60,100,0]
K_to_T_right_bearing = [80,121,80,40,0]
K_to_lc1_right_bearing = [45,18,13,80,0]
K_to_lc4_right_bearing = [86,86,59,100,0]
K_to_lc5_right_bearing = [45,45,27,0,0]
K_to_lc6_right_bearing = [0,98,27,0,0]
K_to_lc7_right_bearing = [100,139,95,0,0]
K_to_lc8_right_bearing = [66,59,54,100,0]

L_to_AX_right_bearing = [0,0,0,40,0]
L_to_BEFR_right_bearing = [0,0,60,40,0]
L_to_DHIKLMNPSUZ_right_bearing = [0,0,0,40,0]
L_to_CGOQ_right_bearing = [0,0,60,120,0]
L_to_J_right_bearing = [259,259,203,60,0]
L_to_T_right_bearing = [207,207,203,200,0]
L_to_VW_right_bearing = [0,0,140,140,0]
L_to_Y_right_bearing = [0,0,0,160,0]
L_to_lc1_right_bearing = [58,27,41,50,0]
L_to_lc2_lc3_right_bearing = [40,40,41,0,0]
L_to_lc4_right_bearing = [93,68,93,100,0]
L_to_lc6_right_bearing = [74,54,52,0,0]
L_to_lc7_right_bearing = [156,122,107,0,0]
L_to_lc8_right_bearing = [57,54,52,40,0]

M_to_AVWXY_right_bearing = [0,0,0,40,0]
M_to_BDEFHIKLMNPRSUZ_right_bearing = [0,0,0,40,0]
M_to_CGOQ_right_bearing = [0,0,0,50,0]
M_to_J_right_bearing = [128,108,128,40,0]
M_to_T_right_bearing = [92,92,92,20,0]
M_to_lc1_right_bearing = [55,17,27,0,0]
M_to_lc2_lc3_right_bearing = [43,27,27,0,0]
M_to_lc4_right_bearing = [66,54,27,0,0]
M_to_lc6_lc7_right_bearing = [66,40,60,0,0]
M_to_lc8_right_bearing = [66,40,27,0,0]

N_to_AVWXY_right_bearing = [0,0,0,40,0]
N_to_BDEFHIKLMNPRTUZ_right_bearing = [0,0,0,40,0]
N_to_CGOQ_right_bearing = [0,0,0,50,0]
N_to_J_right_bearing = [100,80,80,40,0]
N_to_lc1_right_bearing = [41,0,14,0,0]
N_to_lc2_lc3_right_bearing = [41,0,14,0,0]
N_to_lc6_lc7_right_bearing = [41,0,14,0,0]
N_to_lc8_right_bearing = [61,0,41,40,0]

O_to_A_right_bearing = [112,153,112,120,0]
O_to_BDEFHKLNPRU_right_bearing = [0,0,0,40,0]
O_to_I_right_bearing = [0,0,0,60,0]
O_to_J_right_bearing = [112,112,112,50,0]
O_to_M_right_bearing = [64,105,64,40,0]
O_to_VWXYZ_right_bearing = [0,0,0,100,0]
O_to_lc1_right_bearing = [91,91,77,0,0]
O_to_lc2_lc3_right_bearing = [50,40,9,0,0]
O_to_lc4_right_bearing = [41,0,10,0,0]
O_to_lc7_right_bearing = [51,0,10,0,0]
O_to_lc8_right_bearing = [105,91,77,0,0]

P_to_A_right_bearing = [100,100,100,100,0]
P_to_BDEFHKLNPRU_right_bearing = [40,0,0,40,0]
P_to_CGOQ_right_bearing = [40,40,40,70,0]
P_to_I_right_bearing = [0,0,0,70,0]
P_to_J_right_bearing = [200,112,100,200,0]
P_to_M_right_bearing = [120,126,77,40,0]
P_to_S_right_bearing = [50,50,50,0,0]
P_to_T_right_bearing = [0,0,0,90,0]
P_to_VW_right_bearing = [0,0,0,70,0]
P_to_XYZ_right_bearing = [60,60,60,90,0]
P_to_lc1_right_bearing = [135,135,135,100,0]
P_to_lc2_right_bearing = [45,45,0,0,0]
P_to_lc3_right_bearing = [45,45,45,0,0]
P_to_lc4_right_bearing = [60,74,46,0,0]
P_to_lc5_right_bearing = [67,67,67,0,0]
P_to_lc6_right_bearing = [53,53,58,0,0]
P_to_lc7_right_bearing = [80,80,58,0,0]
P_to_lc8_right_bearing = [130,130,135,100,0]

Q_to_AJS_right_bearing = [0,0,0,60,0]
Q_to_BDEFHKLMNPRUZ_right_bearing = [0,0,0,60,0]
Q_to_CGOQ_right_bearing = [0,0,0,60,0]
Q_to_T_right_bearing = [0,0,0,90,0]
Q_to_VW_right_bearing = [0,0,0,100,0]
Q_to_XY_right_bearing = [0,0,0,90,0]
Q_to_lc1_right_bearing = [40,40,26,40,0]
Q_to_lc6_right_bearing = [46,26,0,0,0]
Q_to_lc7_right_bearing = [46,26,0,0,0]
Q_to_lc8_right_bearing = [65,40,26,40,0]

R_to_ABDEFHIKLMNRPXZ_right_bearing = [40,40,40,40,0]
R_to_CGOQ_right_bearing = [40,40,40,110,0]
R_to_J_right_bearing = [150,173,146,110,0]
R_to_SU_right_bearing = [0,0,0,60,0]
R_to_T_right_bearing = [110,104,104,100,0]
R_to_VWY_right_bearing = [40,40,40,120,0]
R_to_lc1_right_bearing = [40,40,40,80,0]
R_to_lc4_right_bearing = [90,104,90,80,0]
R_to_lc6_right_bearing = [54,76,63,0,0]
R_to_lc7_right_bearing = [122,117,107,0,0]
R_to_lc8_right_bearing = [40,40,26,80,0]

S_to_A_right_bearing = [101,128,101,100,0]
S_to_BDEFHIKLNPRU_right_bearing = [40,40,40,40,0]
S_to_CGOQ_right_bearing = [40,40,40,60,0]
S_to_I_right_bearing = [0,0,0,20,0]
S_to_J_right_bearing = [150,124,104,60,0]
S_to_M_right_bearing = [92,147,92,40,0]
S_to_S_right_bearing = [60,60,60,0,0]
S_to_T_right_bearing = [64,64,50,60,0]
S_to_VW_right_bearing = [0,0,0,100,0]
S_to_XY_right_bearing = [108,108,81,100,0]
S_to_Z_right_bearing = [108,108,81,40,0]
S_to_lc1_right_bearing = [84,97,70,0,0]
S_to_lc2_right_bearing = [40,40,40,0,0]
S_to_lc3_right_bearing = [69,83,42,0,0]
S_to_lc4_right_bearing = [94,108,67,40,0]
S_to_lc5_right_bearing = [100,100,100,40,0]
S_to_lc6_lc7_right_bearing = [127,160,100,40,0]
S_to_lc8_right_bearing = [133,97,90,40,0]

T_to_A_right_bearing = [76,160,213,160,0]
T_to_J_right_bearing = [60,160,154,160,0]
T_to_BDEFLR_right_bearing = [-40,40,97,40,0]
T_to_HKNPU_right_bearing = [0,0,0,40,0]
T_to_CGOQ_right_bearing = [0,80,160,140,0]
T_to_I_right_bearing = [-70,-50,40,0,0]
T_to_M_right_bearing = [0,35,145,40,0]
T_to_S_right_bearing = [0,100,160,80,0]
T_to_VW_right_bearing = [0,40,0,60,0]
T_to_XYZ_right_bearing = [-40,40,100,60,0]
T_to_lc1_right_bearing = [110,200,200,180,0]
T_to_lc2_right_bearing = [-40,100,60,0,0]
T_to_lc3_right_bearing = [95,95,170,160,0]
T_to_lc4_right_bearing = [124,124,190,160,0]
T_to_lc5_right_bearing = [95,109,200,160,0]
T_to_lc6_lc7_right_bearing = [96,100,200,160,0]
T_to_lc8_right_bearing = [138,138,200,180,0]

U_to_A_right_bearing = [80,85,80,80,0]
U_to_B_right_bearing = [99,126,58,40,0]
U_to_DEL_right_bearing = [86,86,60,40,0]
U_to_F_right_bearing = [47,47,47,40,0]
U_to_HIKNPSUCGOQ_right_bearing = [0,0,0,40,0]
U_to_R_right_bearing = [68,68,66,40,0]
U_to_XYZ_right_bearing = [68,68,66,60,0]
U_to_M_right_bearing = [80,85,80,40,0]
U_to_VW_right_bearing = [80,85,80,60,0]
U_to_J_right_bearing = [172,172,172,40,0]
U_to_T_right_bearing = [126,153,126,40,0]
U_to_lc1_right_bearing = [46,46,20,0,0]
U_to_lc2_lc3_right_bearing = [49,41,0,0,0]
U_to_lc4_lc5_right_bearing = [83,70,60,0,0]
U_to_lc6_lc7_right_bearing = [124,96,83,0,0]
U_to_lc8_right_bearing = [42,69,73,0,0]

V_to_A_right_bearing = [154,154,154,200,0]
V_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
V_to_CGOQ_right_bearing = [0,0,0,80,0]
V_to_J_right_bearing = [88,74,88,160,0]
V_to_M_right_bearing = [74,88,88,40,0]
V_to_lc1_right_bearing = [85,100,123,100,0]
V_to_lc3_right_bearing = [0,0,0,100,0]
V_to_lc4_lc5_right_bearing = [0,0,0,100,0]
V_to_lc6_lc7_right_bearing = [0,0,0,100,0]
V_to_lc8_right_bearing = [82,100,123,100,0]

W_to_A_right_bearing = [141,141,141,200,0]
W_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
W_to_CGOQ_right_bearing = [0,0,0,80,0]
W_to_J_right_bearing = [88,88,88,160,0]
W_to_M_right_bearing = [63,110,83,40,0]
W_to_lc1_right_bearing = [105,78,105,100,0]
W_to_lc3_right_bearing = [0,0,0,100,0]
W_to_lc4_right_bearing = [0,0,0,100,0]
W_to_lc5_right_bearing = [45,4,18,100,0]
W_to_lc6_lc7_right_bearing = [45,17,20,100,0]
W_to_lc8_right_bearing = [119,78,105,100,0]

X_to_A_right_bearing = [68,68,68,40,0]
X_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
X_to_CGOQ_right_bearing = [0,0,0,100,0]
X_to_J_right_bearing = [100,100,100,100,0]
X_to_M_right_bearing = [68,68,68,40,0]
X_to_lc1_right_bearing = [40,40,13,60,0]
X_to_lc4_right_bearing = [60,70,60,100,0]
X_to_lc5_right_bearing = [60,70,60,20,0]
X_to_lc7_right_bearing = [170,150,87,0,0]
X_to_lc8_right_bearing = [40,40,40,80,0]

Y_to_A_right_bearing = [182,200,237,200,0]
Y_to_BDEFHIKLNPRTUVWXYZ_right_bearing = [40,60,60,40,0]
Y_to_CGOQ_right_bearing = [80,120,100,100,0]
Y_to_J_right_bearing = [150,200,107,160,0]
Y_to_M_right_bearing = [59,100,141,40,0]
Y_to_S_right_bearing = [100,100,100,40,0]
Y_to_T_right_bearing = [59,40,60,40,0]
Y_to_lc1_lc8_right_bearing = [223,220,278,140,0]
Y_to_lc2_right_bearing = [0,60,57,0,0]
Y_to_lc3_right_bearing = [120,120,175,100,0]
Y_to_lc4_right_bearing = [160,133,187,100,0]
Y_to_lc5_right_bearing = [140,144,194,100,0]
Y_to_lc6_lc7_right_bearing = [140,126,180,100,0]

Z_to_A_right_bearing = [72,100,80,60,0]
Z_to_BDEFHIKLNPRUVWXYZ_right_bearing = [0,0,0,40,0]
Z_to_J_right_bearing = [72,100,120,120,0]
Z_to_M_right_bearing = [72,100,80,40,0]
Z_to_CGOQS_right_bearing = [100,100,63,80,0]
Z_to_T_right_bearing = [89,89,89,40,0]
Z_to_lc1_right_bearing = [44,44,44,40,0]
Z_to_lc2_lc3_right_bearing = [58,31,44,0,0]
Z_to_lc4_right_bearing = [80,135,94,60,0]
Z_to_lc5_right_bearing = [44,44,44,0,0]
Z_to_lc6_right_bearing = [114,86,44,60,0]
Z_to_lc7_right_bearing = [155,113,126,60,0]
Z_to_lc8_right_bearing = [90,72,72,40,0]

#######################################################################################################

# Define Functions/Procedures

####################################################################################################### 

def proc_main_plugin(image, layer, font_name_pointer, font_size, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour,
                    text, step_size, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                    position_of_medial, use_space_to_change_step_size, compensate, compensate_multiplier,
                    kern_amount, keep_layers, keep_paths, f_nohook_flag, g_loop_flag, 
                    j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, 
                    z_loop_flag):
    
    #gimp.progress_init("Creating text on path...")
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
	
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #Define variables for font-joining routine
    force_eflag = True
    text = fn_clean_up_text(text)
    #Rem: force_eflag determines what type of letter e should follow an unjoined hooked letter
    #Rem: Looped hook fonts do not require the flag to be set
    force_sflag = False
    #Rem: force_sflag determines whether letter s changes to accomodate joins from looped and unlooped e
    #Rem: Italic fonts do not shape e in different ways, therefore force_sflag must be set to True

    # Get font name from user
    if (font_name_pointer == 0):
        font_name = "ItalicVariations Italic"
        force_sflag = True
    if (font_name_pointer == 1):
        font_name = "ItalicBoldVariations"
        force_sflag = True
    if (font_name_pointer == 2):
        font_name = "ItalicBlackVariations"
        force_sflag = True
    if (font_name_pointer == 3):
        font_name = "JoinerVariations"
    if (font_name_pointer == 4):
        font_name = "SquareBeads"
    
		
    text = text.decode('utf-8') # Allows unicode characters above 127 to be displayed without creating errors
    
    # Get path/vector information
    # num_vectors holds the number of paths associated with the current image
    # vector_ids holds the list of ids for the paths associated with the current image
    num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
    pdb.gimp_selection_none(image)
	
	
    if (num_vectors == 0):
        # No path present so report to user
        old_handler = pdb.gimp_message_get_handler()		
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message("There is no path to work with: \nShowing joined-up text as a single line instead.")	
        pdb.gimp_message_set_handler(old_handler)
        # Convert text to joined-up text
        new_text, name_layer = fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag)
        # Draw the joined-up text 
        proc_show_joined_up_text_line(image, layer, new_text, font_name, font_colour, font_size, 0, 100, name_layer)
    else:

		
        # Convert text to joined-up text
        new_text, name_layer = fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag)
        proc_text_along_path(image, layer, new_text, name_layer, font_name_pointer,
                            font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                            keep_layers, keep_paths, use_space_to_change_step_size, compensate, 
                            compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, position_of_medial)

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
#######################################################################################################

def fn_get_gradient_positions(instances):
    # get positions along gradients for text colours
    grad_position = 0.0
    if instances<1: instances = 1
    grad_amount = float(1.00/instances)
    loop = 0
    while loop<=instances:
        if loop == 0:
            list_of_pos = [grad_position]
        else:
            list_of_pos.append(grad_position)
        grad_position += grad_amount
        loop += 1
    return list_of_pos

#######################################################################################################

def fn_clean_up_text(t_string):
    # replace newline characters from current text with space
    loop,t_flag = 0,0
    n_string = "No text has been entered!"
    while (loop<len(t_string)):
        if (t_string[loop]>" "):
            if (t_flag == 0):
                n_string = t_string[loop]
                t_flag = 1
            else:
                n_string += t_string[loop]
        else:
            if (t_flag>0):
                n_string += " "
                
        loop += 1
    return n_string

#######################################################################################################

def fn_shade_text_colour(f_colour, bl_colour, instances, amount):
    red_amount_old = f_colour[0]
    green_amount_old = f_colour[1]
    blue_amount_old = f_colour[2]
    red_amount_new = bl_colour[0]
    green_amount_new = bl_colour[1]
    blue_amount_new = bl_colour[2]
    zero = 0.0
	
    if red_amount_old<red_amount_new:
        red_spread = float(red_amount_new - red_amount_old)
        red_step = float(red_spread/instances)
    else:
        red_spread = float(red_amount_old - red_amount_new)
        red_step = float(zero - (red_spread/instances))
		
    if green_amount_old<green_amount_new:
        green_spread = float(green_amount_new - green_amount_old)
        green_step = float(green_spread/instances)
    else:
        green_spread = float(green_amount_old - green_amount_new)
        green_step = float(zero - float((green_spread/instances)))

    if blue_amount_old<blue_amount_new:
        blue_spread = float(blue_amount_new - blue_amount_old)
        blue_step = float(blue_spread/instances)
    else:
        blue_spread = float(blue_amount_old - blue_amount_new)
        blue_step = float(zero - float((blue_spread/instances)))

    new_red_amount = red_amount_old + (red_step * amount)
    if new_red_amount < 0: new_red_amount = 0
    if new_red_amount > 255: new_red_amount = 255
	
    new_green_amount = green_amount_old + (green_step * amount)
    if new_green_amount < 0: new_green_amount = 0
    if new_green_amount > 255: new_green_amount = 255
	
    new_blue_amount = blue_amount_old + (blue_step * amount)
    if new_blue_amount < 0: new_blue_amount = 0
    if new_blue_amount > 255: new_blue_amount = 255
	
    new_f_colour = (int(new_red_amount), int(new_green_amount), int(new_blue_amount))

	
    return new_f_colour
		
#######################################################################################################

def fn_shade_gradient_colour(colours_of_samples, instances):
    red_amount = colours_of_samples[(instances*4) + 0] * 255
    green_amount = colours_of_samples[(instances*4) + 1] * 255
    blue_amount = colours_of_samples[(instances*4) + 2] * 255
    alpha_amount = colours_of_samples[(instances*4) + 3] * 255

    if red_amount < 0: red_amount = 0
    if red_amount > 255: red_amount = 255
    if green_amount < 0: green_amount = 0
    if green_amount > 255: green_amount = 255
    if blue_amount < 0: blue_amount = 0
    if blue_amount > 255: blue_amount = 255
    if alpha_amount < 0: alpha_amount = 0
    if alpha_amount > 255: alpha_amount = 255
	
    new_f_colour = (int(red_amount), int(green_amount), int(blue_amount), int(alpha_amount))
    return new_f_colour

#######################################################################################################

def proc_text_along_path(image, layer, joining_text_list, text, font_name_pointer, 
                        f_name, f_colour,  bl_colour, spread_colour, joinedup_gradient, use_gradient_colour, f_size, step_size, kern_amount, 
                        keep_layers, keep_paths, use_space_to_change_step_size, compensation, compensate_multiplier, 
                        use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, position_of_medial):

    governor = 0 # initialise variable to use to prevent creation of unnecessary layers
    attack = 0 # initialise variable to mark point of change in text size within text string when size modifier flag is set
    spaces_loop = 0
    spaces_count = text.count(" ")
    if not use_gradient_colour:
        if spaces_count<2: spaces_count = 2
    target = 0
    original_font_colour = f_colour
    shading_loop = 0
	
    if len(text) % 2 == 0: # variable to help position attack (first) phase of change in text size
        even = True
    else:
        even = False
		
    if " " not in text: use_space_to_change_step_size = False # Force option off, if there are no space characters in the text
		
    if use_text_size_modifier:
        if use_space_to_change_step_size:  
            target = text.count(" ")
            if target<3: 
                use_space_to_change_step_size = False
                
                target = len(text)
        else:
            target = len(text)
            if target<3: use_text_size_modifier = False

    if use_gradient_colour: 
        # get number of colours etc needed for using colours from current ui gradient
        current_gradient = joinedup_gradient
        if use_space_to_change_step_size:
            list_of_gradient_positions = fn_get_gradient_positions(spaces_count)
            number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
        else:
            list_of_gradient_positions = fn_get_gradient_positions(len(text)-1)
            number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
		
    if use_text_size_modifier:

        if target == 3: # ie minimum number of characters/spaces to work with
            f_size = initial_text_size
            attack = 1
        else:
            f_size = initial_text_size
            if use_space_to_change_step_size: 
                if even: target += 0
            attack = int((float(target)/100) * int(position_of_medial)) # set attack to the point in the text string/space count where the font size will change
            if attack>(target - 1): attack = (target - 1)
	
    if use_text_size_modifier: # get step size for attack phase
        size_difference,new_step_size = 0,0
        if medial_text_size > initial_text_size: size_difference = medial_text_size - initial_text_size
        if initial_text_size > medial_text_size: size_difference = initial_text_size - medial_text_size
        if target>3:
            if use_space_to_change_step_size: 
                if even: attack += 0
                if attack < 1: attack = 1
                new_step_size = size_difference/(attack)
            else:
                if even: attack -= 1
                if attack < 1: attack = 1
                new_step_size = size_difference/(attack)
        else:
            new_step_size = size_difference/(attack)
			
        if new_step_size < 0.5: new_step_size = 0.5
        if medial_text_size < initial_text_size: new_step_size = 0 - new_step_size
        step_size = new_step_size
        
    # Get the current active path
    active_vector = pdb.gimp_image_get_active_vectors(image)
    active_vector_name = pdb.gimp_item_get_name(active_vector)
	
    if (active_vector == -1):
        active_vector = pdb.gimp_image_set_active_vectors(image,image.vectors[0])
	
    active_vectors = pdb.gimp_image_get_active_vectors(image)
    # Get path stroke information
    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
    path_length = pdb.gimp_vectors_stroke_get_length(active_vectors,ActivePath_StrokeID,0.01)
	
	# initialise some variables
    f_xpos = 0
    f_ypos = 40
    curve = 0
    curve_compensation = 0
    temp_angle_diff = 0
    # Find the width to use for the space character
    (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
    space_width = fontbox_width
	
    # Check to see if the space character has paths so that we can set a flag if necessary
    temp_space_string = " -"
    temp_space_layer = pdb.gimp_text_layer_new(image,temp_space_string,f_name,f_size,0)
    pdb.gimp_image_insert_layer(image,temp_space_layer,None,-1)
    temp = pdb.gimp_vectors_new_from_text_layer(image, temp_space_layer)
    space_vectors_path = pdb.gimp_image_insert_vectors(image,temp,None,-1)
    space_vectors = pdb.gimp_image_get_vectors_by_name(image, temp_space_string)
    # Get space character path stroke information
    NumberOfStrokes_in_Space_Path,Space_Path_StrokeIDs_List = pdb.gimp_vectors_get_strokes(space_vectors)
    pdb.gimp_image_remove_vectors(image,space_vectors)
    temp = pdb.gimp_image_remove_layer(image, temp_space_layer)

	# Check to see if the font uses a (visible) glyph in the space character to set a flag
    space_has_vectors = False
    if (NumberOfStrokes_in_Space_Path > 1):
        space_has_vectors = True

    text_layer_y_origin = image.height - (f_size + f_ypos)
	
    # Find the initial font size to use for text trails
    if (step_size < 0):
        initial_f_size = f_size + (step_size)
    elif (step_size > 0):
        initial_f_size = (f_size - step_size)
    else:
        initial_f_size = (f_size)
		
    if initial_f_size < 2: initial_f_size = 2
    
    (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname("dummy", initial_f_size, PIXELS, str(f_name))
    font_y_offset = fontbox_ascent

    char_width = 0
    # Calculate kerning to be used between characters, if required by user, based on size of font taken from user settings
    kern_factor = kern_amount/f_size
    kern_gap = kern_factor * f_size

    loop = 0
    # Initialise a loop specifically to use for creating layer and path identifiers and names
    layer_and_path_loop = 0
    #id = pdb.gimp_progress_get_window_handle()
    #pdb.gimp_progress_init("Creating text on path...",id)
    #pdb.gimp_message(str(progress_id))
    #gimp.progress_init("Creating text on path...")
    while loop < len(joining_text_list):
        CurrentChar, RealCurrentChar = joining_text_list[loop], text[loop]
		
        if loop < (len(joining_text_list)-1):
            NextChar, RealNextChar = joining_text_list[loop + 1], text[loop + 1]
        else:
            NextChar, RealNextChar = "", ""

        # reset character x position adjustment factors
        f_xpos_adjustment, j_xpos_adjustment, p_xpos_adjustment, t_xpos_adjustment = 0,0,0,0
        xpos_adjustment = 0
		
        # adjust horizontal positioning for letter f 

        if (NextChar == unichr(102)): # standalone letter f left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(f_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(f_alone_left_bearing[font_name_pointer], f_size)

        if (joining_text_list[loop] == unichr(102)): # standalone letter f right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(f_alone_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(f_alone_right_bearing[font_name_pointer], f_size)
        
        if ((NextChar > unichr(293)) and (NextChar < unichr(299))): #  letter f and alternative f left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(f_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(f_left_bearing[font_name_pointer], f_size)

        if (joining_text_list[loop] > unichr(293) and joining_text_list[loop] < unichr(299)): #  letter f and alternative f right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(f_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(f_right_bearing[font_name_pointer], f_size)
			
        # adjust horizontal positioning for alternative letter f
		
        if (NextChar == unichr(414)): # standalone letter f and standalone alternative f left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(alt_f_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_f_alone_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] == unichr(414)): # standalone letter f and standalone alternative f right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(alt_f_alone_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_f_alone_right_bearing[font_name_pointer], f_size)

        if (NextChar > unichr(414) and NextChar < unichr(420)): #  letter f and alternative f left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(alt_f_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_f_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] > unichr(414) and joining_text_list[loop] < unichr(420)): #  letter f and alternative f right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #f_xpos_adjustment = fn_font_x_pos_adjustment(alt_f_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_f_right_bearing[font_name_pointer], f_size)

        # adjust horizontal positioning for letter j
		
        if (NextChar == unichr(106)): # standalone letter j left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(j_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(j_alone_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] == unichr(106)): # standalone letter j right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment((j_alone_right_bearing[font_name_pointer]), f_size)
                xpos_adjustment += fn_font_x_pos_adjustment((j_alone_right_bearing[font_name_pointer]), f_size)
			
        if ((NextChar > unichr(315) and NextChar < unichr(321))): # join to letter j left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(j_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(j_left_bearing[font_name_pointer], f_size)
				
        if ((joining_text_list[loop] > unichr(315) and joining_text_list[loop] < unichr(321)) ): # letter j right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(j_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(j_right_bearing[font_name_pointer], f_size)

        # adjust horizontal positioning for alternative letter j
		
        if (NextChar == unichr(428)): # standalone alternative letter j left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(alt_j_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_j_alone_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] == unichr(428)): # standalone alternative letter j right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment((alt_j_alone_right_bearing[font_name_pointer]), f_size)
                xpos_adjustment += fn_font_x_pos_adjustment((alt_j_alone_right_bearing[font_name_pointer]), f_size)
			
        if (NextChar > unichr(428) and NextChar < unichr(435)): # join to alternative letter j left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(alt_j_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_j_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] > unichr(428) and joining_text_list[loop] < unichr(435) ): # alternative letter j right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #j_xpos_adjustment = fn_font_x_pos_adjustment(alt_j_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(alt_j_right_bearing[font_name_pointer], f_size)

        # adjust horizontal positioning for letter p
		
        if (NextChar == unichr(112)): # standalone letter p left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #p_xpos_adjustment = fn_font_x_pos_adjustment(p_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(p_alone_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] == unichr(112)): # standalone letter p right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #p_xpos_adjustment = fn_font_x_pos_adjustment(p_alone_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(p_alone_right_bearing[font_name_pointer], f_size)
			
        if (NextChar > unichr(349) and NextChar < unichr(356)): # join to letter p left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #p_xpos_adjustment = fn_font_x_pos_adjustment(p_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(p_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] > unichr(349) and joining_text_list[loop] < unichr(356)): # letter p right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #p_xpos_adjustment = fn_font_x_pos_adjustment(p_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(p_right_bearing[font_name_pointer], f_size)

        # adjust horizontal positioning for letter t

        if (NextChar == unichr(116)): # standalone letter t left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #t_xpos_adjustment = fn_font_x_pos_adjustment(t_alone_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(t_alone_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] == unichr(116)): # standalone letter t right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #t_xpos_adjustment = fn_font_x_pos_adjustment(t_alone_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(t_alone_right_bearing[font_name_pointer], f_size)
			
        if (NextChar > unichr(372) and NextChar < unichr(379)): # join to letter t left bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #t_xpos_adjustment = fn_font_x_pos_adjustment(t_left_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(t_left_bearing[font_name_pointer], f_size)
				
        if (joining_text_list[loop] > unichr(372) and joining_text_list[loop] < unichr(379)): # letter t right bearing adjustment
            if (RealCurrentChar <> RealNextChar):
                #t_xpos_adjustment = fn_font_x_pos_adjustment(t_right_bearing[font_name_pointer], f_size)
                xpos_adjustment += fn_font_x_pos_adjustment(t_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "A": # adjust kerning after A where needed
            if (NextChar in "ABDEFHKLMNPRS"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_ABDEFHKLMNPRS_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "IU"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_IU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar == "Y"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_Y_right_bearing[font_name_pointer], f_size)
            if (NextChar == "Z"):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_Z_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc2_3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(A_to_lc8_right_bearing[font_name_pointer], f_size)

        if CurrentChar == "B": # adjust kerning after B where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFLRS"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_BDEFLRS_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar  in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(B_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "C": # adjust kerning after C where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFLR"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_BDEFLR_right_bearing[font_name_pointer], f_size)
            if (NextChar in "IKMNPU"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_IKMNPU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQS"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_CGOQS_right_bearing[font_name_pointer], f_size)
            if (NextChar in "JT"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_JT_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(C_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "D": # adjust kerning after D where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFLR"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_BDEFLR_right_bearing[font_name_pointer], f_size)
            if (NextChar in "HKNPUCGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_HKNPUCGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(D_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "E": # adjust kerning after E where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQS"):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_CGOQS_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VWMXY"):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_VWMXY_right_bearing[font_name_pointer], f_size)
            if (NextChar in "JT"):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_JT_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(E_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "F": # adjust kerning after F where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEHLNU"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_BDEHLNU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "FIKPRVW"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_FIKPRVW_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(F_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "G": # adjust kerning after G where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLMNPRUS"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_BDEFHIKLMNPRUS_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4 or NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(G_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "H": # adjust kerning after H where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRU"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_BDEFHIKLNPRU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "COGQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_COGQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4 or NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(H_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "I": # adjust kerning after I where needed
            if (NextChar in "AM"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_AM_right_bearing[font_name_pointer], f_size)
            if (NextChar == "B"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_B_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "DEFHIKLNPRUS"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_DEFHIKLNPRUS_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(I_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "J": # adjust kerning after J where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLMNPRTUXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_BDEFHIKLMNPRTUXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQS"):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_CGOQS_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4 or NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(J_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "K": # adjust kerning after K where needed
            if (NextChar in "AUVWXY"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_AUVWXY_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLMNPRTZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_BDEFHIKLMNPRZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(K_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "L": # adjust kerning after L where needed
            if (NextChar in "AX"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_AX_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BEFR"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_BEFR_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "DHIKLMNPSUZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_DHIKLMNPSUZ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar == "Y"):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_Y_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(L_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "M": # adjust kerning after M where needed
            if (NextChar in "AVWXY"):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_AVWXY_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLMNPRSUZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_BDEFHIKLMNPRSUZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(M_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "N": # adjust kerning after N where needed
            if (NextChar in "AVWXY"):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_AVWXY_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLMNPRTUZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_BDEFHIKLMNPRTUZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(N_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "O": # adjust kerning after O where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRU"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_BDEFHKLNPRU_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_VWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(O_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "P": # adjust kerning after P where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHKLNPRU"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_BDEFHKLNPRU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(P_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "Q": # adjust kerning after Q where needed
            if (NextChar in "AJS"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_AJS_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHKLMNPRUZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_BDEFHKLMNPRUZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XY"):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_XY_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(Q_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "R": # adjust kerning after R where needed
            if (NextChar in "ABDEFHIKLMNRPXZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_ABDEFHIKLMNRPXZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar in "SU"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_SU_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VWY"):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_VWY_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(R_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "S": # adjust kerning after S where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRU"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_BDEFHIKLNPRU_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XY"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_XY_right_bearing[font_name_pointer], f_size)
            if (NextChar == "Z"):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_Z_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(S_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "T": # adjust kerning after T where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFLR"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_BDEFLR_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "HKNPU"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_HKNPU_right_bearing[font_name_pointer], f_size)
            if (NextChar == "I"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_I_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(T_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "U": # adjust kerning after U where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar == "B"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_B_right_bearing[font_name_pointer], f_size)
            if (NextChar in "DEL"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_DEL_right_bearing[font_name_pointer], f_size)
            if (NextChar in "HIKNPSUCGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_HIKNPSUCGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "F"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_F_right_bearing[font_name_pointer], f_size)
            if (NextChar == "R"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_R_right_bearing[font_name_pointer], f_size)
            if (NextChar in "XYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_XYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in "VW"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_VW_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4 or NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(U_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "V": # adjust kerning after V where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4 or NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(V_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "W": # adjust kerning after W where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(W_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "X": # adjust kerning after X where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(X_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "Y": # adjust kerning after Y where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRTUVWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_BDEFHIKLNPRTUVWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQ"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_CGOQ_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_J_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "S"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_S_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1 or NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc1_lc8_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc2_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6 or NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(Y_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
				
        if CurrentChar == "Z": # adjust kerning after Z where needed
            if (NextChar == "A"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_A_right_bearing[font_name_pointer], f_size)
            if (NextChar in "BDEFHIKLNPRUVWXYZ"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_BDEFHIKLNPRUVWXYZ_right_bearing[font_name_pointer], f_size)
            if (NextChar in "CGOQS"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_CGOQS_right_bearing[font_name_pointer], f_size)
            if (NextChar == "J"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar == "M"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_M_right_bearing[font_name_pointer], f_size)
            if (NextChar == "T"):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_T_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_1):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc1_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_2 or NextChar in lower_case_3):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_4):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc4_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_5):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc5_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_6):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc6_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_7):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc7_right_bearing[font_name_pointer], f_size)
            if (NextChar in lower_case_8):
                xpos_adjustment += fn_font_x_pos_adjustment(Z_to_lc8_right_bearing[font_name_pointer], f_size)
				
        if (CurrentChar > " " or (CurrentChar == " " and space_has_vectors == True)):
            # Create layers for each character in user defined string		        
            if (loop == 0):
                # Create a list to hold the text layer identifiers
                layer_list = ["text_layer" + str(layer_and_path_loop)]
                # Create a list to hold the text layer names
                layer_name_list = [str(joining_text_list[layer_and_path_loop])]
				
                # Create the layer for the first character
                layer_list[layer_and_path_loop] = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
                # and insert this first layer into the image

                pdb.gimp_image_insert_layer(image,layer_list[layer_and_path_loop],None,-1)
                #pdb.gimp_text_layer_set_kerning(layer_list[layer_and_path_loop],True)
                # Set the new text layer colour
                if use_gradient_colour: f_colour = fn_shade_gradient_colour(colours_of_samples,loop)
                pdb.gimp_text_layer_set_color(layer_list[layer_and_path_loop], f_colour)
                # Rename the new layer 
                pdb.gimp_item_set_name(layer_list[layer_and_path_loop],str(text[loop]))

                # Set the x,y position of the first layer within the image
                pdb.gimp_layer_translate(layer_list[layer_and_path_loop],f_xpos,text_layer_y_origin)

                # note the width of the current character
                char_width = layer_list[layer_and_path_loop].width
				
                offset_x,offset_y = pdb.gimp_drawable_offsets(layer_list[layer_and_path_loop])

                # To insert paths of letters in the text list
                # Create a list to hold the path layer ids
                path_id_list = [pdb.gimp_vectors_new_from_text_layer(image,layer_list[layer_and_path_loop])]
                # Create another list to hold the names of the paths
                paths_names_list = [str(joining_text_list[layer_and_path_loop])]
                # Add the path to the image
                temp = pdb.gimp_image_insert_vectors(image,path_id_list[layer_and_path_loop],None,-1)
                # Get the path id associated with the current character
                char_vector = pdb.gimp_image_get_vectors_by_name(image, paths_names_list[layer_and_path_loop])
                # Get path info for current character path to work out character width
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]
                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = vector_char_width
                offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.

                dist =  f_xpos + float(char_width)/2 #calculate center dist of letter before adding to width.
                # Get the x,y position of the required point along the active path to position the characters
                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)

                translate_x = x_point - float(char_width)/2
                translate_y = y_point - float(layer_list[layer_and_path_loop].height)/2 # ie on the path
                pdb.gimp_layer_translate(layer_list[layer_and_path_loop],translate_x - f_xpos,translate_y - text_layer_y_origin)
                
                angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                # Initialise angle variables to be used in trying to compensate for distortions along curved path
                current_angle, prev_angle = math.degrees(angle), math.degrees(angle)
                x,y = pdb.gimp_drawable_offsets(layer_list[layer_and_path_loop])
                #pdb.gimp_drawable_transform_rotate(layer_list[loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2),TRANSFORM_FORWARD,INTERPOLATION_CUBIC,True,3,TRANSFORM_RESIZE_ADJUST)
                temp = pdb.gimp_item_transform_rotate(layer_list[loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2))
                #pdb.gimp_drawable_transform_rotate(layer_list[loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2),TRANSFORM_FORWARD,INTERPOLATION_CUBIC,True,3,3)
                if (compensation == 1): curve_compensation = fn_compensate_for_curve_in_path(prev_angle, current_angle, f_size)
                if (keep_paths == 0):
                    pdb.gimp_image_remove_vectors(image,path_id_list[layer_and_path_loop])
                layer_and_path_loop += 1

            else:
			
                # Add further layer identifiers to the list of text layer identifiers
                layer_list.append(["text_layer" + str(layer_and_path_loop)])
                # and create each subsequent layer
                layer_list[layer_and_path_loop] = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
                # Insert subsequent layers into the image
                pdb.gimp_image_insert_layer(image,layer_list[layer_and_path_loop],None,-1)
                #pdb.gimp_text_layer_set_kerning(layer_list[layer_and_path_loop],True)
				
                # Set the new text layer colour
                if spread_colour and not use_gradient_colour: 
                    if not use_space_to_change_step_size:
                        f_colour = fn_shade_text_colour(original_font_colour, bl_colour, len(text)-1, loop)
                    else:
                        f_colour = fn_shade_text_colour(original_font_colour, bl_colour, spaces_count - 1, shading_loop)
						
                if use_gradient_colour: 
                    if use_space_to_change_step_size: 
                        #pdb.gimp_message("shading_loop = " + str(shading_loop))
                        f_colour = fn_shade_gradient_colour(colours_of_samples,shading_loop)
                    else:
                        f_colour = fn_shade_gradient_colour(colours_of_samples,loop)
						
                pdb.gimp_text_layer_set_color(layer_list[layer_and_path_loop], f_colour)
                # Rename the layer
                pdb.gimp_item_set_name(layer_list[layer_and_path_loop],str(text[loop]))
				
                # Get width of current character
                char_width = layer_list[layer_and_path_loop].width
                # Get the name of the current layer
                temp = str(pdb.gimp_item_get_name(layer_list[layer_and_path_loop]))

                # Add further layer names to the list of text layer names
                layer_name_list.append(temp)

                # Rename the layer to account for multiple occurences of the same letter
                pdb.gimp_item_set_name(layer_list[layer_and_path_loop],str(layer_name_list[layer_and_path_loop]))
                # Set x,y the position of subsequent layers in the image
				
                pdb.gimp_layer_translate(layer_list[layer_and_path_loop],f_xpos,text_layer_y_origin )

                # Add further path identifiers to the list of path identifiers
                path_id_list.append(pdb.gimp_vectors_new_from_text_layer(image,layer_list[layer_and_path_loop]))
                # Add further names of the list paths names
                # Use loop instead of layer_and_path_loop since we are referencing the original string for the path layer 
                paths_names_list.append(str(joining_text_list[loop]))
                # Insert path
                temp = pdb.gimp_image_insert_vectors(image,path_id_list[layer_and_path_loop],None,-1)
                char_vector = pdb.gimp_image_get_vectors_by_name(image, paths_names_list[layer_and_path_loop])
				
                NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                firstX = ListOfControlPoints[2]
                secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                minX = min(firstX,secondX)
                maxX = max(firstX,secondX)
                vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                add_width = vector_char_width
                offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.

                dist =  f_xpos + float(char_width)/2 #calculate center dist of letter before adding to width.

                # Get the x,y position of the required point along the active path to position the characters
                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
                angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                current_angle = math.degrees(angle)
			
                if (compensation == 1): 
                    curve_compensation,temp_angle_diff = fn_compensate_for_curve_in_path(prev_angle, current_angle, f_size)
                    curve_compensation = (curve_compensation * compensate_multiplier)
                    curve = curve_compensation
				
                dist =  (f_xpos + curve_compensation) + float(char_width)/2 #calculate center dist of letter before adding to width.

                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
				
				
                #angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                #current_angle = math.degrees(angle)
			
                translate_x = x_point - float(char_width)/2
                translate_y = y_point - float(layer_list[layer_and_path_loop].height)/2 # ie on the path
                pdb.gimp_layer_translate(layer_list[layer_and_path_loop],translate_x - f_xpos,translate_y - text_layer_y_origin)
			
                #gimp.progress_init("Creating text on path...")
				
				
				
                x,y = pdb.gimp_drawable_offsets(layer_list[layer_and_path_loop])
                #pdb.gimp_drawable_transform_rotate(layer_list[layer_and_path_loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2),TRANSFORM_FORWARD,INTERPOLATION_CUBIC,True,3,TRANSFORM_RESIZE_ADJUST)
                temp = pdb.gimp_item_transform_rotate(layer_list[layer_and_path_loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2))
				
                #pdb.gimp_drawable_transform_rotate(layer_list[layer_and_path_loop],angle,False,x+float(char_width)/2,y+float(layer_list[layer_and_path_loop].height/2),TRANSFORM_FORWARD,INTERPOLATION_CUBIC,True,3,0)
				
                # Delete path
                if (keep_paths == 0):
                    pdb.gimp_image_remove_vectors(image,path_id_list[layer_and_path_loop])
                # Merge new character layer to first character layer
                if (keep_layers == 0):
                    #pdb.gimp_image_merge_down(image, layer_list[layer_and_path_loop],EXPAND_AS_NECESSARY)
                    pdb.gimp_image_merge_down(image, layer_list[layer_and_path_loop],0)
					
                temp = pdb.gimp_displays_flush()

                layer_and_path_loop += 1
				

        # Update y position adjustment
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(CurrentChar, f_size, PIXELS, str(f_name))
        text_layer_y_origin = text_layer_y_origin + (font_y_offset - fontbox_ascent)
        font_y_offset = fontbox_ascent
		
        #f_xpos = f_xpos + fontbox_width + kern_gap + curve - f_xpos_adjustment - j_xpos_adjustment - p_xpos_adjustment - t_xpos_adjustment
        f_xpos = f_xpos + fontbox_width + kern_gap + curve - xpos_adjustment

        prev_angle = current_angle

        # Stop font size becoming too small/large
        if (CurrentChar == " "):
            if (use_space_to_change_step_size == 1):
                f_size_adjustment = step_size
                f_size = f_size + f_size_adjustment
                if (f_size<2):
                    f_size = 2
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if (f_size>3000):
                    f_size = 3000
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                kern_gap += (f_size * kern_factor)
                shading_loop += 1
            if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                if use_space_to_change_step_size:
                    if attack == spaces_loop:
                        decay = target - attack
                        if not even: decay -= 1
                        if not even: f_size -= (step_size * 1)
                        if even: f_size -= (step_size * 2)
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
                        if target>3:
                            new_step_size = size_difference/(decay)
                        else:
                            new_step_size = size_difference/(decay)
                        if new_step_size < 0.5: new_step_size = 0.5
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
                    spaces_loop += 1
                else:
				
                    if attack == loop:
                        decay = target - attack
                        if even: decay -=2
                        if not even: f_size -= (step_size * 1)
                        if even: f_size -= (step_size)
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
                        if target>3:
                            new_step_size = size_difference/(decay)
                        else:
                            new_step_size = size_difference/(decay)
                        if new_step_size < 0.5: new_step_size = 0.5
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
						


        else:
            if (use_space_to_change_step_size == 0):
                f_size_adjustment = step_size
                f_size = f_size + f_size_adjustment
                if (f_size<2):
                    f_size = 2
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if (f_size>3000):
                    f_size = 3000
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                    if attack == loop:
                        decay = len(text) - attack
                        if even: 
                            decay -=2 
                        else:
                            decay -= 0
							
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
						
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
						
                        if len(text)>3:
                            new_step_size = size_difference/(decay)
                        else:
                            new_step_size = size_difference/(decay)
							
                        if not even: 
                            if final_text_size > medial_text_size:
						        f_size = medial_text_size + new_step_size
                            else:
                                f_size -= (step_size + (new_step_size * 2))
						   
                        if even: f_size -= (step_size)
                        if new_step_size < 0.5: new_step_size = 0.5
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
                kern_gap = (f_size * kern_factor)
        
        #pdb.gimp_progress_pulse()
        #gimp.progress_update(float(1.00/len(joining_text_list)*loop))
        #gimp.progress_update(float(1.00/len(joining_text_list)*loop))
        (tempbox_width, tempbox_height, tempbox_ascent, tempbox_descent) = pdb.gimp_text_get_extents_fontname(NextChar, f_size, PIXELS, str(f_name))
        if (f_xpos + tempbox_width >= path_length): loop = len(joining_text_list)
        loop = loop + 1
    
    if (keep_layers == 0):
        final_layer = pdb.gimp_image_get_active_layer(image)
        pdb.gimp_item_set_name(final_layer,"Flattened path text")
	#pdb.gimp_progress_end()
	
#####################################################################################################
	
def fn_compensate_for_curve_in_path(p_angle, c_angle, f_size):
    curve_compensation,angle_diff = 0,0
	
    if (c_angle<0 and p_angle>180):
        c_angle += 360
    if (c_angle>180 and p_angle<0):
        c_angle -= 360
	
    if (c_angle>p_angle): # tests for text along path travelling in a clockwise direction
        
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = (min_angle - max_angle)
        curve_compensation = fn_font_x_pos_adjustment(angle_diff, f_size)
                

    if (c_angle<p_angle): # tests for  text along path travelling in a counter-clockwise direction
						
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = max_angle - min_angle
        curve_compensation = fn_font_x_pos_adjustment(angle_diff,f_size)
		
    return curve_compensation,angle_diff

		
#######################################################################################################

def proc_show_joined_up_text_line(image, layer, text, f_name, f_colour, f_size, f_xpos, f_ypos, name_layer):
    layer_height = layer.height
    text_layer = pdb.gimp_text_layer_new(image, text, f_name, f_size, 0)
    pdb.gimp_image_insert_layer(image,text_layer,None,-1)
    pdb.gimp_text_layer_set_color(text_layer, f_colour)
    pdb.gimp_layer_translate(text_layer,f_xpos,image.height - (text_layer.height + f_ypos))
    pdb.gimp_item_set_name(text_layer, name_layer)

#######################################################################################################

def computeThetaWithSlope(fromPoint,toPoint,slope):
    dX=toPoint[0]-fromPoint[0]
    dY=toPoint[1]-fromPoint[1]

    if math.fabs(dX) < .001:
        theta=math.copysign(math.pi/2,dY)
    else:
        theta=math.atan(slope)
        if dX<0:
            theta=theta+math.pi
    return theta

#######################################################################################################

def fn_font_x_pos_adjustment(kerning, f_size):
    adjust_factor = (((100 * kerning) / 1024) * f_size) / 100
    return adjust_factor
	
#######################################################################################################

def fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag):

    # Start of conversion: text 

    TextLen = 0
    AllTextLen = 0
    Horizontal = ""
    Diagonal = ""
    NotJoined = ""
    PreviousChar = ""
    PreviousCharType = ""
    CurrentChar = ""
    CurrentCharType = ""
    NextChar = ""
    NextCharType = ""
    strTemp = ""
    ToRound = ""
    ToSlanted = ""
    ToStraight = ""
    ToSmall = ""
    ToTall = ""
	
    if (x_rounded_flag == True):
        HasHorizontal = "forvw" # define the letters which use horizontal joining strokes
        HasDiagonal = "abcdegjhijklmnpqstuxyz" # define the letters which use diagonal joining strokes
    else:
        HasHorizontal = "forvwx" # define the letters which use horizontal joining strokes
        HasDiagonal = "abcdegjhijklmnpqstuyz" # define the letters which use diagonal joining strokes

    HasNoJoin = " "  # define the lower case letters which have no joining strokes
    GoesToRound = "acdgoqs"
    GoesToSlanted = "vwxz"
    GoesToStraight = "bfhijklmnprtuy"
    GoesToSmall = "ijmnpruvwxyz"
    GoesToTall = "bfhklt"
    Horiz = "horizontal"
    Diagon = "diagonal"
    NoJoin = "notjoined"

    AllTextLen = len(text)
    TextLen = AllTextLen


    # Create a list to hold all the original characters of the text string
    # This list will remain unaltered
    loop = 0
    text_list = [text[loop]]
    loop = loop + 1
    while loop < len(text):
        if (text[loop] < " "):
            text_list.append(" ")
        else:
            text_list.append(text[loop])
        loop = loop + 1

    # Make a copy of the list to work with
    # This list will contain the individual characters of joined-up unicode text
    joining_text_list = text_list

    loop = 1
    NextChar = text_list[loop - 1]
    testing = ""
    NextCharType = NoJoin
    PreviousCharType = NoJoin
    CurrentCharType = NoJoin
    TestHasHorizontal = NextChar in HasHorizontal
    if (TestHasHorizontal == True):
        NextCharType = Horiz
        debug = NextCharType

    TestHasDiagonal = NextChar in HasDiagonal
    if (TestHasDiagonal == True):
        NextCharType = Diagon
    	
    if (TextLen>0):

        while (loop<=TextLen):
            testing = testing + joining_text_list[loop - 1]
            PreviousChar = CurrentChar
            PreviousCharType = CurrentCharType
            CurrentChar = NextChar
            CurrentCharType = NextCharType

            if (loop == TextLen):
                NextCharType = NoJoin
                NextChar = ""
            else:
                NextChar = text_list[loop]
                TestHasNoJoin = NextChar in HasNoJoin
                TestHasHorizontal = NextChar in HasHorizontal
                TestHasDiagonal = NextChar in HasDiagonal
                if (NextChar < "a" or NextChar > "z"):
                    NextCharType = NoJoin
                elif (TestHasNoJoin == True):
                    NextCharType = NoJoin
                elif (TestHasHorizontal == True):
                    NextCharType = Horiz
                else:
                    if (TestHasDiagonal == True):
                        NextCharType = Diagon
            # Deal with smart quote characters
            if (CurrentChar=="\"") and ((PreviousChar==" " or PreviousChar=="")):
                CurrentChar = unichr(8220)
            if (CurrentChar=="\'") and ((PreviousChar==" " or PreviousChar=="")):
                CurrentChar = unichr(8216)


            if ((CurrentChar < "a" or CurrentChar > "z") or loop == TextLen):
				
                if (loop == TextLen):

                # Force final character where alternative character is used
                    if (PreviousCharType == Diagon and CurrentChar == "e"):
                        CurrentChar = unichr(280)
                        if (force_eflag == True):
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(281)
                    
                    if (PreviousCharType == Horiz and CurrentChar == "e"):
                        CurrentChar = unichr(281)
						
                    if (f_nohook_flag == True):
                        if (CurrentChar == "f"):
                            CurrentChar = unichr(414)

                    if (g_loop_flag == True):
                        if (CurrentChar == "g"):
                            CurrentChar = unichr(421)
							
                    if (j_loop_flag == True):
                        if (CurrentChar == "j"):
                            CurrentChar = unichr(428)
							
                    if (k_unloop_flag == True):
                        if (CurrentChar == "k"):
                            CurrentChar = unichr(435)
                    
                    if (CurrentChar == "o"):
                        CurrentChar = unichr(347)
						
                    if (s_closed_flag == True):
                    # Rem Deal with previous horizontal join characters to closed s
                        if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # Check for closed s joins from letters f, o, r, v, w
                            CurrentChar = unichr(449)
                        else:                    
                            if (CurrentChar == "s"):
                                CurrentChar = unichr(442)

                    if (x_rounded_flag == True):
                        if (CurrentChar == "x"):
                            CurrentChar = unichr(456)

                    if (y_loop_flag == True):
                        if (CurrentChar == "y"):
                            CurrentChar = unichr(463)

                    if (z_loop_flag == True):
                        if (CurrentChar == "z"):
                            CurrentChar = unichr(470)									
							
					
                joining_text_list[loop - 1] = CurrentChar

			
            else:
                TestGoesToTall = NextChar in GoesToTall
                TestGoesToSmall = NextChar in GoesToSmall
                TestGoesToRound = NextChar in GoesToRound
                TestGoesToSlanted = NextChar in GoesToSlanted
                TestGoesToStraight = NextChar in GoesToStraight
                TestHasNoJoin = True #NextChar in HasNoJoin
                if (CurrentCharType == Diagon):
                    if (NextChar == "e"):
                        #Select e loop join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(256)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(262)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(268)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(274)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(282)
                            if (PreviousCharType == NoJoin): 
                                CurrentChar = unichr(288)
                            else:
                                if (PreviousCharType == Horiz):
                                    CurrentChar = unichr(288)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(422) # looped g to e
                            else:
                                CurrentChar = unichr(299) # unlooped g to e
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(304)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(310)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(429) # looped j to e
                            else:
                                CurrentChar = unichr(316) # unlooped j to e
                        elif (CurrentChar == "k"):
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(436) # unlooped k to e
                            else:
                                CurrentChar = unichr(321) # looped k to e
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(327)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(333)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(339)
                        elif (CurrentChar == "p"):
                            CurrentChar = unichr(350)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(356)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(443) # closed s to e
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # Check for closed s joins from letters f, o, r, v, w
                                    CurrentChar = unichr(450)
                            else:
                                CurrentChar = unichr(367) # open s to e
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(373)
                        elif (CurrentChar == "u"):
                            CurrentChar = unichr(379)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(457)
                        elif (CurrentChar == "y"):  
                            if (y_loop_flag == True):
                                CurrentChar = unichr(464) # looped y to e
                            else:
                                CurrentChar = unichr(400) # unlooped y to e
                        else:
                            if (CurrentChar == "z"):  
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(471) # looped z to e
                                else:
                                    CurrentChar = unichr(405) # unlooped z to e

                    if (TestGoesToRound == True):
                        # Select rounded join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(257)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(256)
                                if (force_sflag == True):
                                    CurrentChar = unichr(260)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(260)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(263)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(262)
                                if (force_sflag == True):
                                    CurrentChar = unichr(266)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(266)
                        elif (CurrentChar == "c"):
                            CurrentChar = unichr(269)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(268)
                                if (force_sflag == True):
                                    CurrentChar = unichr(272)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(272)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(275)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(274)
                                if (force_sflag == True):
                                    CurrentChar = unichr(278)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(278)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(283)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(282)
                                if (force_sflag == True):
                                    CurrentChar = unichr(286)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(286)
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(289)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(288)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(292)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(292)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(289)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(288)
                                    #if (force_sflag == True):
                                        #CurrentChar = unichr(289)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(292)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(423) # looped g to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(422)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(426)
                            else:
                                CurrentChar = unichr(300) # unlooped g to round
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(430) # looped j to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(429)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(433)
                            else:
                                CurrentChar = unichr(317) # unlooped j to round
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(305)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(304)
                                if (force_sflag == True):
                                    CurrentChar = unichr(308)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(308)
                                #if (force_sflag == True):
                                    #CurrentChar = unichr(308)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(311)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(310)
                                if (force_sflag == True):
                                    CurrentChar = unichr(314)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(314)
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(437) # unlooped k to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(436)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(440)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(440)
                            else:
                                CurrentChar = unichr(322) # looped k to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(321)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(325)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(325)
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(328)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(327)
                                if (force_sflag == True):
                                    CurrentChar = unichr(331)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(331)
                        elif (CurrentChar == "m"):
                            CurrentChar = unichr(334)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(333)
                                if (force_sflag == True):
                                    CurrentChar = unichr(337)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(337)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(340)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(339)
                                if (force_sflag == True):
                                    CurrentChar = unichr(343)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(343)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(351)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(350)
                                if (force_sflag == True):
                                    CurrentChar = unichr(354)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(354)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(357)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(356)
                                if (force_sflag == True):
                                    CurrentChar = unichr(360)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(360)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(444) # closed s to round
                                if (NextChar == "s"):
                                    CurrentChar = unichr(447)
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # horizontal joins to closed s
                                    CurrentChar = unichr(451)
                                    if (NextChar == "s"):
                                        CurrentChar = unichr(454)
                            else:
                                CurrentChar = unichr(368) # open s to round
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(371)
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(374)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(373)
                                if (force_sflag == True):
                                    CurrentChar = unichr(377)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(377)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(380)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(379)
                                if (force_sflag == True):
                                    CurrentChar = unichr(383)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(383)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(458)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(457)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(461)
                                if (force_sflag == True):
                                    CurrentChar = unichr(461)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(465) # looped y to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(464)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(468)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(468)
                            else:
                                CurrentChar = unichr(401) # unlooped y to round
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(472) # looped z to round
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(471)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(475)
                                else:
                                    CurrentChar = unichr(406) # unlooped z to round
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(405)
                                        if (force_sflag == True):
                                            CurrentChar = unichr(409)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(409)
                    if (TestGoesToSlanted == True):
                        # Select slanted join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(258)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(261)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(264)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(267)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(270)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(273)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(276)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(279)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(284)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(287)
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(290)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(293)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(290)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(293)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(424) # looped g to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(427)
                            else:
                                CurrentChar = unichr(301) # unlooped g to slanted
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(306)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(309)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(312)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(315)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(431) # looped j to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(434)
                            else:
                                CurrentChar = unichr(318) # unlooped j to slanted
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(438) # unlooped k to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(441)
                            else:
                                CurrentChar = unichr(323) # looped k to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(326)
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(329)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(332)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(335)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(338)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(341)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(344)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(352)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(355)
                        elif (CurrentChar == "q"): 
                            CurrentChar = unichr(358)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(361)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(445) # closed s to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(448)
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # slanted joins to closed s
                                    CurrentChar = unichr(452)
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(455)
                                if (NextChar == "s"):
                                    CurrentChar = unichr(445)
                            else:
                                CurrentChar = unichr(369) # open s to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(372)
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(375)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(378)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(381)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(384)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(459)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(462)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(466) # looped y to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(469)
                            else:
                                CurrentChar = unichr(402) # unlooped y to slanted
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(473) # looped z to slanted
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(476)
                                else:
                                    CurrentChar = unichr(407) # unlooped z to slanted
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(410)
                    if (TestGoesToStraight == True):
                        # Select straight join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(259)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(265)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(271)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(277)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(285)
                            if (PreviousCharType == NoJoin): 
                                CurrentChar = unichr(291)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(291)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(425) # looped g to straight
                            else:
                                CurrentChar = unichr(302) # unlooped g to straight
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(307)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(313)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(432) # looped j to straight
                            else:
                                CurrentChar = unichr(319) # unlooped j to straight
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(439) # unlooped k to straight
                            else:
                                CurrentChar = unichr(324) # looped k to straight
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(330)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(336)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(342)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(353)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(359) #Use this join for q to u
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(446) # closed s to straight
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # slanted joins to closed s
                                    CurrentChar = unichr(453)
                                if (NextChar == "s"):
                                    CurrentChar = unichr(446)
                            else:
                                CurrentChar = unichr(370) # open s to straight
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(376)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(382)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(460)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(467) # looped y to straight
                            else:
                                CurrentChar = unichr(403) # unlooped y to straight
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(474) # looped z to straight
                                else:
                                    CurrentChar = unichr(408) # unlooped z to straight

                     
                    if (TestHasNoJoin == True):
                        if (CurrentChar == "e"):
                            if (PreviousCharType == Diagon): 
                                CurrentChar = unichr(280)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(281)
                
                if (CurrentCharType == Horiz):
                    if (NextChar == "e"):
                        # Select e loop join for all horizontal characters
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(415) # unhooked f to e
                            else:
                                CurrentChar = unichr(294) # hooked f to e
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(345)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(362)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(385)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(390)
                        else:
                            if (CurrentChar == "x"):
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(457) # rounded x to e
                                else:
                                    CurrentChar = unichr(395) # straight x to e
                    elif (TestGoesToRound == True):
                        # Select rounded join for all horizontal characters
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(416) # unhooked f to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(415)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(419)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(419)
                            else:
                                CurrentChar = unichr(295) # hooked f to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(294)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(298)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(298)
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(346)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(345)
                                if (force_sflag == True):
                                    CurrentChar = unichr(349)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(349)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(363)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(362)
                                if (force_sflag == True):
                                    CurrentChar = unichr(366)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(366)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(386)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(385)
                                if (force_sflag == True):
                                    CurrentChar = unichr(389)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(389)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(391)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(390)
                                if (force_sflag == True):
                                    CurrentChar = unichr(394)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(394)
                        else:
                            if (CurrentChar == "x"): 
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(458) # rounded x to rounded
                                else:
                                    CurrentChar = unichr(396) # straight x to rounded
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(395)
                                        if (force_sflag == True):
                                            CurrentChar = unichr(399)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(399)
                    elif (TestGoesToSmall == True):
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(417) # unhooked f to slanted
                            else:
                                CurrentChar = unichr(296) # hooked f to slanted
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(347)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(364)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(387)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(392)
                        else:
                            if (CurrentChar == "x"): 
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(459) # rounded x to small
                                else:
                                    CurrentChar = unichr(397) # straight x to small
                    else:
                        if (TestGoesToTall == True):
                            if (CurrentChar == "f"): 
                                if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                    CurrentChar = unichr(418) # unhooked f to straight
                                else:
                                    CurrentChar = unichr(297) # hooked f to straight
                            elif (CurrentChar == "o"): 
                                CurrentChar = unichr(348)
                            elif (CurrentChar == "r"): 
                                CurrentChar = unichr(365)
                            elif (CurrentChar == "v"): 
                                CurrentChar = unichr(388)
                            elif (CurrentChar == "w"): 
                                CurrentChar = unichr(393)
                            else:
                                if (CurrentChar == "x"):
                                    if (x_rounded_flag == True): # forcing straight x to rounded x
                                        CurrentChar = unichr(460) # rounded x to tall
                                    else:
                                        CurrentChar = unichr(398) # straight x to tall
                        if (TestHasNoJoin == True):
                            if (CurrentChar == "o"): 
                                CurrentChar = unichr(347)

                if (force_eflag == True):
                    #Deal with letter e after an unjoined or joined descender
                    if (CurrentChar == unichr(282)): # joined looped e to e
                        if (PreviousChar == unichr(299)): # unjoined g to e
                            CurrentChar = unichr(288) # unjoined e to e
                        if (PreviousChar == unichr(316)): # unjoined j to e
                            CurrentChar = unichr(288) # unjoined e to e
                        if (PreviousChar == unichr(400)): # unjoined y to e
                            CurrentChar = unichr(288) # unjoined e to e
                    if (CurrentChar == unichr(283)): # joined looped e to round letter
                        if (PreviousChar == unichr(299)): # unjoined g to e
                            CurrentChar = unichr(289) # unjoined e to round letter
                        if (PreviousChar == unichr(316)): # unjoined j to e
                            CurrentChar = unichr(289) # unjoined e to round letter
                        if (PreviousChar == unichr(400)): # unjoined y to e
                            CurrentChar = unichr(289) # unjoined e to round letter
                    if (CurrentChar == unichr(284)): # joined looped e to slanted letter
                        if (PreviousChar == unichr(299)): # unjoined g to e
                            CurrentChar = unichr(290) # unjoined e to slanted
                        if (PreviousChar == unichr(316)): # unjoined j to e
                            CurrentChar = unichr(290) # unjoined e to slanted
                        if (PreviousChar == unichr(400)): # unjoined y to e
                            CurrentChar = unichr(290) # unjoined e to slanted
                    if (CurrentChar == unichr(285)): # joined looped e to straight
                        if (PreviousChar == unichr(299)): # unjoined g to e
                            CurrentChar = unichr(291) # unjoined e to straight
                        if (PreviousChar == unichr(316)): # unjoined j to e
                            CurrentChar = unichr(291) # unjoined e to straight
                        if (PreviousChar == unichr(400)): # unjoined y to e
                            CurrentChar = unichr(291) # unjoined e to straight
                    if (CurrentChar == unichr(280)): # looped e unjoined
                        if ((PreviousChar == unichr(299)) or (PreviousChar == unichr(300)) or (PreviousChar == unichr(301))  or (PreviousChar == unichr(302)) ): # all unjoined g letters
                            CurrentChar = unichr(281) # unlooped e unjoined
                        if ((PreviousChar == unichr(316)) or (PreviousChar == unichr(317)) or (PreviousChar == unichr(318))  or (PreviousChar == unichr(319) )): # all unjoined j letters
                            CurrentChar = unichr(281) # unlooped e unjoined
                        if ((PreviousChar == unichr(400)) or (PreviousChar == unichr(401)) or (PreviousChar == unichr(402))  or (PreviousChar == unichr(403)) ): # all unjoined y letters
                            CurrentChar = unichr(281) # unlooped e unjoined

                if (f_nohook_flag == True):
                    if (CurrentChar == "f"):
                        CurrentChar = unichr(414) # force ordinary hooked f to unhooked f
                if (g_loop_flag == True):
                    if (CurrentChar == "g"):
                        CurrentChar = unichr(421) # force ordinary unlooped g to looped g
                if (j_loop_flag == True):
                    if (CurrentChar == "j"):
                        CurrentChar = unichr(428) # force ordinary unlooped j to looped j
                if (k_unloop_flag == True):
                    if (CurrentChar == "k"):
                        CurrentChar = unichr(435) # force ordinary looped k to unlooped k
                if (s_closed_flag == True):
                    if (CurrentChar == "s"):
                        CurrentChar = unichr(442) # force ordinary open s to closed s
                        if (PreviousChar == unichr(294) or PreviousChar == unichr(345) or PreviousChar == unichr(362) or PreviousChar == unichr(385) or PreviousChar == unichr(390) or PreviousChar == unichr(395) or PreviousChar == unichr(415)): # Check for closed s joins from letters f, o, r, v, w
                            CurrentChar = unichr(449)
                if (x_rounded_flag == True):
                    if (CurrentChar == "x"):
                        CurrentChar = unichr(456) # force ordinary straight x to rounded x
                if (y_loop_flag == True):
                    if (CurrentChar == "y"):
                        CurrentChar = unichr(463) # force ordinary unlooped y to looped y
                if (z_loop_flag == True):
                    if (CurrentChar == "z"):
                        CurrentChar = unichr(470) # force ordinary unlooped z to looped z
						
            joining_text_list[loop - 1] = CurrentChar
			
            loop = loop + 1
		

    loop = 0	
    joined_text = ""
    while (loop < len(joining_text_list)):
        joined_text = joined_text + joining_text_list[loop]
        loop = loop + 1
    label_text = ""
    loop = 0
    while (loop < len (text)):
        if (text[loop] >= " "):
            label_text += text[loop]

        loop += 1
	
    return joined_text, label_text

#######################################################################################################

# Plugin Registration and User Interface Settings

#######################################################################################################


register(
    "python_fu_JoinedUp_Text_v2-08",
    "To emulate joined up writing ",
    "To emulate joined up writing using customised fonts ",
    "KGH",
    "Open source (BSD 3-clause license)",
    "2017",
    "<Image>/Filters/Text FX/Joined-up Text...",
    "*",
    [
    (PF_OPTION, "pf_joined_fonts", "Joining Font Name:", 0, ("ItalicVariations", "ItalicBoldVariations", "ItalicBlackVariations", "JoinerVariations","SquareBeads")),
    (PF_SPINNER, "pf_fontsize", "Multiline Font size:", 200, (1, 3000, 1)),
    (PF_COLOR, "pf_color", "Text colour:", (0.0, 0.0, 0.0)),
    (PF_COLOR, "pf_color2", "Shading colour:", (255, 255, 255)),
    (PF_TOGGLE, "pf_shade_text", "Shade colour along text:", 0),
    (PF_GRADIENT, "pf_current_gradient", "Gradient for text colours:", 0),
    (PF_TOGGLE, "pf_gradient_text", "Colour text from\ncurrent gradient:", 0),
    (PF_TEXT, "text", "Multiline Text:", "A bobby dazzler!"),
    (PF_SPINNER, "pf_text_size_factor", "Text size step change:", -2,(-100,100,0.5)),
    (PF_TOGGLE, "pf_use_text_size_modifier", "Use text size modifiers:", 0),
    (PF_SPINNER, "pf_initial_text_size", "Initial text size:", 100,(2,3000,1)),
    (PF_SPINNER, "pf_medial_text_size", "Target text size:", 200,(2,3000,1)),
    (PF_SPINNER, "pf_final_text_size", "Final text size:", 100,(2,3000,1)),
    (PF_SLIDER, "pf_position_of_target", "Target for change in text\nsize as percentage along path:", 50,(1,99,5)),
    (PF_TOGGLE, "pf_space_alters_size", "Change size with words:", 0),
    (PF_TOGGLE, "pf_compensate_for_curve", "Compensate for changes \nin curved paths:", 1),
    (PF_SPINNER, "pf_compensate_multiplier", "Compensation multiplier:", 3,(1,10,0.5)),
    (PF_SPINNER, "pf_kerning_factor", "Adjust spacing \nbetween characters:", 0,(-500,500,1)),
    (PF_TOGGLE, "pf_retain_layers", "Retain layers:", 0),
    (PF_TOGGLE, "pf_retain_paths", "Retain paths:", 0),
    (PF_TOGGLE, "pf_force_unhooked_f", "Alternative letter f:", 0),
    (PF_TOGGLE, "pf_force_looped_g", "Alternative letter g:", 0),
    (PF_TOGGLE, "pf_force_looped_j", "Alternative letter j:", 0),
    (PF_TOGGLE, "pf_force_unlooped_k", "Alternative letter k:", 0),
    (PF_TOGGLE, "pf_force_closed_s", "Alternative letter s:", 0),
    (PF_TOGGLE, "pf_force_rounded_x", "Alternative letter x:", 0),
    (PF_TOGGLE, "pf_force_looped_y", "Alternative letter y:", 0),
    (PF_TOGGLE, "pf_force_looped_z", "Alternative letter z:", 0),
    ],
    [],
    proc_main_plugin)

main()


